#!/usr/bin/env python

"A WebStack application for a system configurator."

import WebStack.Generic
import XSLForms.Fields
import XSLForms.Output
import XSLForms.Prepare
import XSLOutput
import libxml2dom
import os

# NOTE: Move these functions into a common utility library.

def add_elements(positions, element_name, element_parent_name=None):
    if not positions:
        return
    for position in positions:
        if element_parent_name:
            parent_elements = position.xpath(element_parent_name)
            if not parent_elements:
                parent_element = position.ownerDocument.createElementNS(None, element_parent_name)
                position.appendChild(parent_element)
            else:
                parent_element = parent_elements[0]
        else:
            parent_element = position
        parent_element.appendChild(position.ownerDocument.createElementNS(None, element_name))

def remove_elements(positions):
    if not positions:
        return
    for position in positions:
        position.parentNode.removeChild(position)

class ConfiguratorResource:

    "A resource providing a system configurator."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    encoding = "utf-8"

    def respond(self, trans):

        global XSLForms # NOTE: Strangely required to avoid UnboundLocalError!

        # Only obtain field information according to the stated method.

        method = trans.get_request_method()
        path_info = trans.get_path_info()

        if method == "GET":
            fields = XSLForms.Fields.Fields(encoding="iso-8859-1", values_are_lists=1)
            parameters = trans.get_fields_from_path()
            documents = fields.make_documents(parameters.items())
        elif method == "POST":
            fields = XSLForms.Fields.Fields(encoding=self.encoding, values_are_lists=1)
            if path_info == "/cpu":
                text = trans.get_request_stream().read()
                parameters = {}
                for text_line in text.split("\r\n"):
                    text_parts = text_line.split("=")
                    text_name, text_values = text_parts[0], ["=".join(text_parts[1:])]
                    parameters[text_name] = text_values
            else:
                parameters = trans.get_fields_from_body(self.encoding)
            documents = fields.make_documents(parameters.items())
            print "*", libxml2dom.toString(documents["configuration"])
        else:
            trans.set_response_code(405)
            raise WebStack.Generic.EndOfResponse

        # Ensure the presence of a document.

        if documents.has_key("configuration"):
            configuration = documents["configuration"]
        else:
            configuration = fields.new_instance("configuration")

        # Add and remove elements according to the selectors found.

        selectors = fields.get_selectors(parameters.items(), documents)
        add_elements(selectors.get("add-memory-unit"), "memory-unit")
        remove_elements(selectors.get("remove-memory-unit"))
        add_elements(selectors.get("add-storage-unit"), "storage-unit")
        remove_elements(selectors.get("remove-storage-unit"))
        add_elements(selectors.get("add-hard-disk"), "hard-disk")
        remove_elements(selectors.get("remove-hard-disk"))

        # Send a response according to certain parameters.
        # When exported, an XML version of the data is returned.

        if parameters.has_key("export"):
            trans.set_content_type(WebStack.Generic.ContentType("text/xml", self.encoding))
            libxml2dom.toStream(configuration, trans.get_response_stream(), trans.get_response_stream_encoding())

        # When not exported, the data is transformed to produce a normal Web
        # page.

        else:

            # Transform, adding enumerations/ranges.

            database_xsl = os.path.join(self.resource_dir, "config_database.xsl")
            database_xml = os.path.join(self.resource_dir, "config_database.xml")
            proc = XSLOutput.Processor([database_xsl], references={"database" : database_xml})
            configuration = proc.get_result(configuration)

            # Start the response.

            trans.set_content_type(WebStack.Generic.ContentType("text/html", self.encoding))

            # Define the stylesheet parameters.

            stylesheet_parameters = {}

            # Ensure that an output stylesheet exists.

            trans_xsl = os.path.join(self.resource_dir, "config_output.xsl")
            template_xml = os.path.join(self.resource_dir, "config_template.xhtml")
            XSLForms.Prepare.ensure_stylesheet(template_xml, trans_xsl)

            if path_info == "/cpu":
                trans_xsl = os.path.join(self.resource_dir, "config_output_cpu.xsl")
                template_xml = os.path.join(self.resource_dir, "config_output.xsl")
                XSLForms.Prepare.ensure_stylesheet_fragment(template_xml, trans_xsl, "cpu-node")
                target_field_name = parameters.get("target-field-name", [""])[0]
                print "*", target_field_name
                stylesheet_parameters["element-path"] = XSLForms.Output.get_element_path(target_field_name)
                print "*", stylesheet_parameters["element-path"]

            stylesheet_parameters["application-url"] = \
                "http://%s:%s%s" % (trans.get_server_name(), trans.get_server_port(), trans.get_path_without_query())

            # Complete the response.

            proc = XSLOutput.Processor([trans_xsl], parameters=stylesheet_parameters)
            proc.send_output(trans.get_response_stream(), trans.get_response_stream_encoding(),
                configuration)

            import sys
            proc = XSLOutput.Processor([trans_xsl], parameters=stylesheet_parameters)
            proc.send_output(sys.stderr, "iso-8859-1", configuration)

# vim: tabstop=4 expandtab shiftwidth=4
